<?php
defined('SYSPATH') or die('No direct access allowed.');

require_once('op5/license/op5lic.php');

/**
 * Logger controller
 *
 * op5, and the op5 logo are trademarks, servicemarks, registered servicemarks
 * or registered trademarks of op5 AB.
 * All other trademarks, servicemarks, registered trademarks, and registered
 * servicemarks mentioned herein may be the property of their respective
 * owner(s).
 * The information contained herein is provided AS IS with NO WARRANTY OF ANY
 * KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY, AND FITNESS FOR A
 * PARTICULAR PURPOSE.
 */
class Logger_Controller extends Base_reports_Controller {
	public $type = 'logger';

	public function __construct() {
		parent::__construct();

		$l = new op5lic();
		if (!$l->query(array('logserver' => 1))) {
			Router::$method = 'no_license';
		} else {
			$auth = Auth::instance();
			if (!$auth->authorized_for('logger_schedule_archive_search')) {
				Router::$method = 'unauthorized';
			}
		}
	}

	public function no_license() {
		$this->template->content = $this->add_view('logger/no_license');
	}

	public function unauthorized() {
		header('HTTP/1.1 403 Forbidden');
		$this->template->content = $this->add_view('403');
		$this->template->title = _('Unauthorized');
	}

	public function index($input = false) {
		$this->setup_options_obj($input);
		$this->template->content = $this->add_view('reports/setup');
		$template = $this->template->content;
		$template->report_options = $this->add_view('logger/options');
		$template->saved_reports = $this->options->get_all_saved();
		$template->scheduled_info = Scheduled_reports_Model::report_is_scheduled($this->type, $this->options['report_id']);
	}

	public function generate($input = false) {
		$this->setup_options_obj($input);

		$start_time = strtotime($this->options['start_time']);
		$end_time = strtotime($this->options['end_time']);

		$this->template->content = $this->add_view('reports/index');
		$this->template->content->report_options = $this->add_view('logger/options');
		$this->template->content->header = $this->add_view('reports/header');
		$this->template->content->header->report_time_formatted = sprintf("From <strong>%s</strong> to <strong>%s</strong>", date(nagstat::date_format(), $start_time), date(nagstat::date_format(), $end_time));
		$this->template->content->header->skip_csv = true;
		$this->template->title = $this->template->content->header->title = _('Log archive search');
		$this->template->content->content = $this->add_view("logger/archive");
		$this->template->content->content->drop_output_buffers = true; // Hack for monitor 7.0
		$template = $this->template->content->content;
		$this->template->js[] = $this->add_path('reports/js/common.js');
		$this->template->js_strings = reports::js_strings();

		$template->error = array();
		$template->query = false;
		$template->columns = false;
		$template->start_time = $start_time;
		$template->end_time = $end_time;

		if(!$start_time) {
			$template->error[] = "The start_time you've entered cannot be interpreted as a point in time. Please check the <a href=\"http://php.net/manual/en/datetime.formats.php\">documentation</a>.";
		}
		if(!$end_time) {
			$template->error[] = "The end_time you've entered cannot be interpreted as a point in time. Please check the <a href=\"http://php.net/manual/en/datetime.formats.php\">documentation</a>.";
		}

		if (!$template->error) {
			try {
				$logset = ObjectPool_Model::get_by_query($this->options['query']);
				/* @var $logset LogMessageSet_Model */
				$logset = $logset->reduce_by('rtime', $start_time, '>=');
				$logset = $logset->reduce_by('rtime', $end_time, '<=');

				if($logset instanceof LogMessageSet_Model) {
					/* Get columns in frontend */
					$fecolumns = $logset->get_all_columns_list();

					/* id doesn't make sense in archived data */
					$fecolumns = array_diff($fecolumns, array ('id'));

					/* Convert column names to backend */
					$becolumns = array_map(
						function ($name) use($logset) {
							return $logset->map_name_to_backend($name, '');
						}, $fecolumns);

					/* Filter out virtual columns */
					$becolumns = array_filter($becolumns);

					$template->query = $logset->generate_logfilter_query($becolumns);
					$template->columns = $becolumns;

					$result = Database::instance('logger')->query("select storage_path from settings")->offsetGet(0);
					if ($result->storage_path) {
						$template->paths = array(
							$result->storage_path,
							'/var/log/op5/logserver/'
						);
					}
					else {
						$template->set = $logset;
					}
					unset($result);
				} else {
					$template->error[] = "Logger archive can only run queries against the log_messages table";
				}
			} catch(ORMException $e) {
				$template->error[] = "There was a problem running your query - please check that it is valid. Internal ORM error: ".$e->getMessage();
			} catch(LSFilterException $e) {
				$template->error[] = "There was a problem running your query - please check that it is valid. Internal LSFilter error: ".$e->getMessage();
			}
		}

		// Since we're not going to do anything more with the session, we can save and close it. That makes subsequent requets nonblocking.
		session_write_close();

		if ($this->options['output_format'] == 'pdf') {
			$this->template->content->content->drop_output_buffers = false; // Hack for monitor 7.0
			return $this->generate_pdf();
		}
	}

	public static function _helptexts($id)
	{
		$helptexts = array(
			'start_time' => _('This delimits the duration for which the query will be executed. You can enter any time expression that would be legal to send to the date() listview function, for example <code>midnight yesterday</code>, <code>5 minutes ago</code> or <code>2012-05-28 12:30</code>. See a <a href="http://php.net/manual/en/datetime.formats.php">full reference</a>.'),
			'end_time' => _('This delimits the duration for which the query will be executed. You can enter any time expression that would be legal to send to the date() listview function, for example <code>midnight yesterday</code>, <code>5 minutes ago</code> or <code>2012-05-28 12:30</code>. See a <a href="http://php.net/manual/en/datetime.formats.php">full reference</a>.'),
			'query' => _('The listview filter on the log_messages table to run the report against - for example, <code>[log_messages] in "my saved filter"</code>'),
		);
		if (array_key_exists($id, $helptexts))
			echo $helptexts[$id];
		else
			parent::_helptexts($id);

	}
}
