<?php
defined('SYSPATH') or die('No direct access allowed.');

/* Hack for 7.0, where everything still is buffered, but we don't want to, so drop buffers, but to not create any errors later, we need to recreate the buffers at the end.  */
$output_buffer_depth = 0;
if ($drop_output_buffers) {
	$output_buffer_depth = ob_get_level();
	Kohana::close_buffers(true);
	echo Kohana::$output; //Since close_buffers uses end_clean, and sets $output
	$output_buffer_depth -= ob_get_level();
}


if ($error) {
	echo "<div id=\"response\"><div class=\"alert error\"><h3>Couldn't generate your archived log search:</h3><ul class=\"error\">";
	foreach ($error as $err) {
		print '<li>'.$err.'</li>';
	}
	echo "</ul></div></div>";
}

if($query !== false) {
	$date_format = nagstat::date_format();
	$FAC_VALUES = LogMessage_Model::$FAC_VALUES;
	$SEV_VALUES = LogMessage_Model::$SEV_VALUES;

	$column_defs = array(
		'ip' => array(
			'name' => _('Source IP address')
		),
		'rtime' => array(
			'name' => _('Receive time'),
			'format' => function($v) use ($date_format) {
				return date($date_format, $v);
			}
		),
		'facility' => array(
			'name' => _('Facility'),
			'format' => function($v) use ($FAC_VALUES) {
				if(!isset($FAC_VALUES[$v]))
					return "Unknown ($v)";
				return $FAC_VALUES[$v];
			}
		),
		'severity' => array(
			'name' => _('Severity'),
			'format' => function($v) use ($SEV_VALUES) {
				if(!isset($SEV_VALUES[$v]))
					return "Unknown ($v)";
				return $SEV_VALUES[$v];
			}
		),
		'mtime' => array(
			'name' => _('Message time'),
			'format' => function($v) use ($date_format) {
				return date($date_format, $v);
			}
		),
		'host' => array(
			'name' => _('Host')
		),
		'ident' => array(
			'name' => _('Ident')
		),
		'pid' => array(
			'name' => _('PID')
		),
		'event' => array(
			'name' => _('Event')
		),
		'msg' => array(
			'name' => _('Message')
		),
	);

	$col_formatters = array();

	print '<table class="report_log">';
	print '<thead>';
	print '<tr>';
	$pdf_hidden = Kohana::config('logger.pdf_columns');
	foreach ($columns as $col) {
		if(isset($column_defs[$col])) {
			$formatter =& $column_defs[$col];
			$name = $formatter['name'];
			if(!isset($formatter['format'])) {
				$formatter['format'] = 'htmlentities';
			}

			if (!in_array($col, $pdf_hidden))
				$formatter['hide_in_pdf'] = true;

			$col_formatters[] = $formatter;
			print '<th class="'.$col.($col=='msg'?' restricted-output':'').(isset($formatter['hide_in_pdf'])?' print-unimportant':'').'">' . htmlentities($name) . '</th>';
		} else {
			$col_formatters[] = null;
		}
	}
	print '</tr>';
	print '</thead>';
	print '<tbody>';
	if($drop_output_buffers) {
		ob_flush();
		flush();
	}

	if (!isset($paths)) {
		foreach ($set->it($columns, array('rtime asc')) as $logline) {
			print '<tr>';
			foreach ($logline->export() as $key => $value) {
				if (isset($column_defs[$key])) {
					if (isset($column_defs[$key]['format']))
						$format_val = $column_defs[$key]['format']($value);
					else
						$format_val = htmlentities($value);
					print '<td'.(isset($column_defs[$key]['hide_in_pdf'])?' class="print-unimportant"':'').'>' . $format_val . '</td>';
				}
			}
			print '</tr>';
			if($drop_output_buffers) {
				ob_flush();
				flush();
			}
		}
	}
	else {
		$binary = Kohana::config('logger.logfilter.path');

		$descspec = array (0 => array ('pipe','r'),1 => array ('pipe','w'),
			2 => array ('pipe','w'));

		$cmd = $binary;
		foreach($paths as $path) {
			$cmd .=  ' --path ' . escapeshellarg($path);
		}
		$cmd .= ' interactive';
		$cmd .= ' --begin ' . escapeshellarg($start_time);
		$cmd .= ' --end ' . escapeshellarg($end_time);
		$cmd .= ' ' . escapeshellarg($query);

		$proc = proc_open($cmd, $descspec, $pipes);

		if ($proc !== false) {
			while (false !== ($line = fgets($pipes[1]))) {
				set_time_limit(30);
				$logline = json_decode($line);
				if (is_array($logline)) {
					print '<tr>';
					foreach ($logline as $i => $value) {
						if ($col_formatters[$i] != null) {
							$format_val = $col_formatters[$i]['format']($value);
							print '<td'.(isset($col_formatters[$i]['hide_in_pdf'])?' class="print-unimportant"':'').'>' . $format_val . '</td>';
						}
					}
					print '</tr>';
				} else {
					print
						"<tr><td colspan=" . count($columns) . ">ERROR: can't parse line: " .
						htmlentities($line) . "</td></tr>";
				}
				if($drop_output_buffers) {
					ob_flush();
					flush();
				}
			}
		}

		fclose($pipes[0]);
		fclose($pipes[1]);

		fpassthru($pipes[2]);
		fclose($pipes[2]);
		proc_close($proc);
	}

	print '</tbody>';
	print '</table>';
}

if ($output_buffer_depth > 0) {
	/* Hack for 7.0: Reopen output buffers */
	while( $output_buffer_depth-- > 1 ) {
		ob_start();
	}
	// We need to hack the rest into kohana buffering... *sigh*
	ob_start(array('Kohana', 'output_buffer'));
}
